<?php

/**
 * Registration class.
 *
 * @category   apps
 * @package    registration
 * @subpackage libraries
 * @author     ClearCenter <developer@clearcenter.com>
 * @copyright  2011-2015 ClearCenter
 * @license    http://www.clearcenter.com/app_license ClearCenter license
 * @link       http://www.clearcenter.com/support/documentation/clearos/registration/
 */

///////////////////////////////////////////////////////////////////////////////
// N A M E S P A C E
///////////////////////////////////////////////////////////////////////////////

namespace clearos\apps\registration;

///////////////////////////////////////////////////////////////////////////////
// B O O T S T R A P
///////////////////////////////////////////////////////////////////////////////

$bootstrap = getenv('CLEAROS_BOOTSTRAP') ? getenv('CLEAROS_BOOTSTRAP') : '/usr/clearos/framework/shared';
require_once $bootstrap . '/bootstrap.php';

///////////////////////////////////////////////////////////////////////////////
// T R A N S L A T I O N S
///////////////////////////////////////////////////////////////////////////////

clearos_load_language('registration');

///////////////////////////////////////////////////////////////////////////////
// D E P E N D E N C I E S
///////////////////////////////////////////////////////////////////////////////

// Classes
//--------

use \clearos\apps\base\Configuration_File as Configuration_File;
use \clearos\apps\base\Country as Country;
use \clearos\apps\base\File as File;
use \clearos\apps\base\Folder as Folder;
use \clearos\apps\base\Product as Product;
use \clearos\apps\base\Shell as Shell;
use \clearos\apps\base\Yum as Yum;
use \clearos\apps\clearcenter\Rest as Rest;
use \clearos\apps\events\Event_Utils as Event_Utils;
use \clearos\apps\mode\Mode_Engine as Mode_Engine;
use \clearos\apps\mode\Mode_Factory as Mode_Factory;
use \clearos\apps\network\Network as Network;
use \clearos\apps\suva\Suva as Suva;
use \clearos\apps\tasks\Cron as Cron;

clearos_load_library('base/Configuration_File');
clearos_load_library('base/Country');
clearos_load_library('base/File');
clearos_load_library('base/Folder');
clearos_load_library('base/Product');
clearos_load_library('base/Shell');
clearos_load_library('base/Yum');
clearos_load_library('clearcenter/Rest');
clearos_load_library('events/Event_Utils');
clearos_load_library('mode/Mode_Engine');
clearos_load_library('mode/Mode_Factory');
clearos_load_library('network/Network');
clearos_load_library('suva/Suva');
clearos_load_library('tasks/Cron');

// Exceptions
//-----------

use \clearos\apps\base\Engine_Exception as Engine_Exception;
use \clearos\apps\base\Validation_Exception as Validation_Exception;

clearos_load_library('base/Engine_Exception');
clearos_load_library('base/Validation_Exception');

///////////////////////////////////////////////////////////////////////////////
// C L A S S
///////////////////////////////////////////////////////////////////////////////

/**
 * Registration for ClearCenter class.
 *
 * @category   apps
 * @package    registration
 * @subpackage libraries
 * @author     ClearCenter <developer@clearcenter.com>
 * @copyright  2011-2015 ClearCenter
 * @license    http://www.clearcenter.com/app_license ClearCenter license
 * @link       http://www.clearcenter.com/support/documentation/clearos/registration/
 */

class Registration extends Rest
{
    ///////////////////////////////////////////////////////////////////////////////
    // C O N S T A N T S
    ///////////////////////////////////////////////////////////////////////////////

    const FILE_CONFIG = '/etc/clearos/registration.conf';
    const FILE_REGISTERED_FLAG = '/var/clearos/registration/registered';
    const FILE_VERIFIED_REPO_FLAG = '/var/clearos/registration/verified';
    const FILE_SDN_NOTICE = '/var/clearos/registration/sdn_notification';
    const FILE_BUILD_ID = '/var/clearos/base/build.id';
    const FILE_HOST_ID = '/var/clearos/base/host.id';
    const FILE_AUDIT = 'audit.json';
    const FOLDER_REGISTRATION = '/var/clearos/registration';
    const COMMAND_CAT = '/bin/cat';
    const COMMAND_DMIDECODE = '/usr/sbin/dmidecode';
    const REGISTER_NEW = 0;
    const REGISTER_EXISTING = 1;
    const CODE_SYSTEM_REGISTERED = 0;
    const CODE_SYSTEM_NOT_REGISTERED = 3;
    const NON_VERIFIED_COMMUNITY_REPOS = 'clearos-centos clearos-centos-updates clearos-epel clearos-updates';

    ///////////////////////////////////////////////////////////////////////////////
    // V A R I A B L E S
    ///////////////////////////////////////////////////////////////////////////////

    ///////////////////////////////////////////////////////////////////////////////
    // M E T H O D S
    ///////////////////////////////////////////////////////////////////////////////

    /**
     * Registration constructor.
     */

    function __construct()
    {
        clearos_profile(__METHOD__, __LINE__);
        parent::__construct();
    }

    /**
     * Register system to the SDN.
     *
     * @param String $username          account username
     * @param String $password          account password
     * @param int    $registration_type registration type (new install or upgrade/reinstall)
     * @param String $system_name       system name
     * @param int    $system            system ID
     * @param int    $subscription      subscription ID
     * @param String $environment       environment
     *
     * @return Object JSON-encoded response
     *
     * @throws Webservice_Exception
     */

    public function register($username, $password, $registration_type, $system_name, $system, $subscription, $environment)
    {
        clearos_profile(__METHOD__, __LINE__);

        try {

            $extras = array (
                'username' => $username, 'password' => $password, 'registration_type' => $registration_type, 'sync_hostkey' => TRUE,
                'system_name' => $system_name, 'system' => $system, 'subscription' => $subscription, 'environment' => $environment,
                'hardware_id' => $this->_get_hardware_id()
            );

            $extras['hardware_id'] = $this->_get_hardware_id();
            $extras['build_id'] = $this->_get_build_id();
            $extras['host_id'] = $this->_get_host_id();
            if (!isset($this->config['opt_out_data']) || !$this->config['opt_out_data'])
                $extras['system_info'] = $this->_get_dmi_system_info();

            $result = $this->request('registration', 'register', $extras);
            $response = json_decode($result);
            if ($response->code == 0) {
                $suva = new Suva();
                $suva->set_device_name($response->device_id);
                if ($registration_type == self::REGISTER_EXISTING)
                    $suva->set_hostkey($response->hostkey);
                $this->set_local_registration_status(TRUE);
                $this->delete_cache();
                // SDN knows about registration type and create dates to determine if we should
                // disable the non-verified repositories
                if ($response->disable_community_repos) {
                    $yum = new Yum();
                    $yum->set_enabled(explode(' ', self::NON_VERIFIED_COMMUNITY_REPOS), FALSE);
                }
            }
            return $result;
        } catch (Exception $e) {
            throw new Webservice_Exception(clearos_exception_message($e), CLEAROS_ERROR);
        }
    }

    /**
     * Updates local registration flag.
     *
     * @param boolean $status true if registered
     *
     * @return void
     *
     * @throws Webservice_Exception
     */

    public function set_local_registration_status($status)
    {
        clearos_profile(__METHOD__, __LINE__);

        try {

            $file = new File(self::FILE_REGISTERED_FLAG);
            if (!$file->exists() && $status)
                $file->create('root', 'root', '0644');
            else if ($file->exists() && !$status)
                $file->delete();
        } catch (Exception $e) {
            // Ignore?
        }
    }

    /**
     * Set the SDN notice file with relevant message.
     *
     * @param array $response response from SDN
     *
     * @return void
     *
     * @throws Webservice_Exception
     */

    public function set_sdn_notice($response)
    {
        clearos_profile(__METHOD__, __LINE__);

        try {
            $file = new File(self::FILE_SDN_NOTICE);

            if ((int)$response['code'] == 0) {
                if ($file->exists())
                    $file->delete();
            } else if ((int)$response['code'] >= 1000) {
                if (!$file->exists())
                    $file->create('webconfig', 'webconfig', '0640');
                $file->add_lines(json_encode($response) . "\n");
            }
        } catch (Exception $e) {
            // Ignore?
        }
    }

    /**
     * Get registration info related to an account on SDN.
     *
     * @param string  $username username
     * @param string  $password password
     *
     * @return Object JSON-encoded response
     * @throws Webservice_Exception
     */

    public function get_registration_info($username = '', $password = '')
    {
        clearos_profile(__METHOD__, __LINE__);

        try {
    
            $extras = array('username' => $username, 'password' => $password);

            $result = $this->request('registration', 'get_registration_info', $extras);

            return $result;
        } catch (Exception $e) {
            throw new Webservice_Exception(clearos_exception_message($e), CLEAROS_ERROR);
        }
    }

    /**
     * Get SDN notice.
     *
     * @return array
     *
     */

    public function get_sdn_notice()
    {
        clearos_profile(__METHOD__, __LINE__);

        try {
            $file = new File(self::FILE_SDN_NOTICE);

            if (!$file->exists())
                return array();

            $contents = end($file->get_contents_as_array());
            $data = json_decode($contents, TRUE);

            if ($data != NULL) {
                if ($data['persistence'] == 'one_time')
                    $file->delete();
                return $data;
            }
            return array();
        } catch (Exception $e) {
            return array();
        }
    }

    /**
     * Get system registration info related to a device on SDN.
     *
     * @param boolean $realtime set realtime to TRUE to fetch real-time data
     *
     * @return Object JSON-encoded response
     * @throws Webservice_Exception
     */

    public function get_system_info($realtime = FALSE)
    {
        clearos_profile(__METHOD__, __LINE__);

        try {
    
            $cachekey = __CLASS__ . '-' . __FUNCTION__; 

            if (!$realtime && $this->_check_cache($cachekey))
                return $this->cache;
    
            $result = $this->request('registration', 'get_system_info');

            $response = json_decode($result);

            // Only set local registration flag if code is specific
            if ($response->code == self::CODE_SYSTEM_NOT_REGISTERED)
                $this->set_local_registration_status(FALSE);
            elseif ($response->code == self::CODE_SYSTEM_REGISTERED)
                $this->set_local_registration_status(TRUE);

            $this->_save_to_cache($cachekey, $result);

            return $result;
        } catch (Exception $e) {
            throw new Webservice_Exception(clearos_exception_message($e), CLEAROS_ERROR);
        }
    }

    /**
     * Check username availability.
     *
     * @param String $username          account username
     *
     * @return Object JSON-encoded response
     * @throws Webservice_Exception
     */

    public function check_username_availability($username)
    {
        clearos_profile(__METHOD__, __LINE__);

        try {
    
            $extras = array('username' => $username);

            $result = $this->request('registration', 'check_username_availability', $extras);

            return $result;
        } catch (Exception $e) {
            throw new Webservice_Exception(clearos_exception_message($e), CLEAROS_ERROR);
        }
    }

    /**
     * Get terms of service.
     *
     *
     * @return Object JSON-encoded response
     * @throws Webservice_Exception
     */

    public function terms_of_service()
    {
        clearos_profile(__METHOD__, __LINE__);

        try {
            $result = $this->request('registration', 'terms_of_service');

            return $result;
        } catch (Exception $e) {
            throw new Webservice_Exception(clearos_exception_message($e), CLEAROS_ERROR);
        }
    }

    /**
     * Create an account on the SDN.
     *
     * @param String  $username  account username
     * @param String  $password  account password
     * @param String  $email     email
     * @param String  $country   country code
     * @param boolean $mailer    subscribe to mailing list
     * @param int     $interests interest groups
     *
     * @return Object JSON-encoded response
     * @throws Webservice_Exception
     */

    public function create_account($username, $password, $email, $country, $mailer, $interests)
    {
        clearos_profile(__METHOD__, __LINE__);

        try {
    
            $extras = array(
                'username' => $username, 'password' => $password,
                'email' => $email, 'country' => $country, 'mailer' => $mailer, 'interests' => $interests
            );

            $result = $this->request('registration', 'create_account', $extras);

            return $result;
        } catch (Exception $e) {
            throw new Webservice_Exception(clearos_exception_message($e), CLEAROS_ERROR);
        }
    }

    /**
     * Returns a list of registration types.
     *
     * @return array
     */

    function get_registration_options()
    {
        clearos_profile(__METHOD__, __LINE__);
            
        $options = Array(
            self::REGISTER_NEW => lang('registration_new'),
            self::REGISTER_EXISTING => lang('registration_existing')
        );
        return $options;
    }

    /**
     * Returns a list of environment types.
     *
     * @return array
     */

    function get_environment_options()
    {
        clearos_profile(__METHOD__, __LINE__);

        $product = new Product();
        if ($product->is_home()) {
            $options = array(
                0 => lang('base_select'),
                'home' => lang('registration_home'),
                'soho' => lang('registration_soho'),
                'other' => lang('registration_other')
            );
        } else {
            $options = array(
                0 => lang('base_select'),
                'home' => lang('registration_home'),
                'soho' => lang('registration_soho'),
                'smb' => lang('registration_smb'),
                'business' => lang('registration_business'),
                'smb_multi' => lang('registration_smb_multi'),
                'edu' => lang('registration_edu'),
                'nfp' => lang('registration_nfp'),
                'gov' => lang('registration_gov'),
                'other' => lang('registration_other')
            );
        }
        return $options;
    }

    /**
     * Returns a list of country codes.
     *
     * @return array
     */

    function get_country_options()
    {
        clearos_profile(__METHOD__, __LINE__);
            
        $country = new Country();
        return $country->get_list();
    }

    /**
     * Reset registration.
     *
     * @return void
     * @throws Engine_Exception
     */

    function reset()
    {
        clearos_profile(__METHOD__, __LINE__);
            
        $suva = new Suva();
        $suva->reset_hostkey();
        $suva->set_device_name(0);
        $this->delete_cache();

        $this->set_local_registration_status(FALSE);

        // Since registration is being forcibly reset, we need to make sure the
        // Marketplace yum plugin will take care of the verified vs. non-verified repo access.
        // Make ure the flag is reset
        $file = new File(self::FILE_VERIFIED_REPO_FLAG, TRUE);
        if ($file->exists())
            $file->delete();
    }

    /**
     * Registration check-in.
     *
     * @return array
     * @throws Engine_Exception
     */

    function check_in()
    {
        clearos_profile(__METHOD__, __LINE__);
            
        if (!$this->is_loaded)
            $this->_load_config();

        try {
            $cron = new Cron();

            $app = 'app-registration';

            $schedule = NULL;
            if ($cron->exists_configlet($app))
                $schedule = $cron->get_configlet($app);
            
            if ($schedule == NULL || preg_match('/\d+ \d+ \* \* \d+.*|0 0 \* \* \*.*/', $schedule)) {
                // Randomize future check-ins
                $cron_entry = rand(0,59) . ' ' . rand(0,23) . ' * * * root /usr/sbin/clearcenter-checkin >/dev/null 2>&1';
                $cron->delete_configlet($app);
                $cron->add_configlet($app, $cron_entry);
                return;
            }
        } catch (Exception $e) {
            // Don't really care
        }

        try {

            // Device ID
            //----------
            $suva = new Suva();
            $extras['device_id'] = $suva->get_device_name();

            // System Mode
            //------------
            if (clearos_library_installed('mode/Mode_Engine')) {
                try {
                    $mode_object = Mode_Factory::create();
                    $extras['system_mode'] = $mode_object->get_mode();
                } catch (\Exception $e) {
                    // Not really worried about
                }
            }

            // Network Mode
            //-------------
            if (clearos_library_installed('network/Network')) {
                try {
                    $network = new Network();
                    $extras['network_mode'] = $network->get_mode();
                } catch (\Exception $e) {
                    // Not really worried about
                }
            }
            // Uptime
            //-------
            $shell = new Shell();
            $shell->execute(self::COMMAND_CAT, '/proc/uptime', FALSE);
            $line = $shell->get_last_output_line();
            if (preg_match('/([0-9.]+)\s+([0-9.])/', $line, $match))
                $extras['uptime'] = $match[1];

            // Hardware ID/Hash
            //-----------------
            $extras['hardware_id'] = $this->_get_hardware_id();
            $extras['build_id'] = $this->_get_build_id();
            $extras['host_id'] = $this->_get_host_id();
            if (!isset($this->config['opt_out_data']) || !$this->config['opt_out_data'])
                $extras['system_info'] = $this->_get_dmi_system_info();

            // Locale and Version Info always get sent up in Rest calls
            // from ClearCenter API

            // Opt-Out data
            //-------------
            try {
                $file = new File(CLEAROS_TEMP_DIR . '/' . self::FILE_AUDIT);
                if ($file->exists()) {
                    $audit = json_decode($file->get_contents());
                    if ($audit != NULL) {
                        if (!isset($this->config['opt_out_data']) || !$this->config['opt_out_data']) {
                            // Users
                            $extras['user'] = $audit->users->weekly;
                            // Unique IP
                            $extras['ip4'] = $audit->ip4->weekly;
                            // Unique MAC
                            $extras['mac'] = $audit->mac->weekly;
                        }
                    }
                }
            } catch (\Exception $e) {
                // Not really worried about
            }
            
            $response = json_decode($this->request('registration', 'check_in', $extras), TRUE);

            // Only set local registration flag if code is specific
            if ($response['code'] == self::CODE_SYSTEM_NOT_REGISTERED) {
                $this->set_local_registration_status(FALSE);
                Event_Utils::add_event(lang('registration_system_unregistered'), 'CRIT', 'REGISTRATION_UNREGISTERED', 'registration', TRUE);
            } elseif ($response['code'] == self::CODE_SYSTEM_REGISTERED) {
                $this->set_local_registration_status(TRUE);
                Event_Utils::resolve_event('REGISTRATION_UNREGISTERED');
            }

            return $response;
        } catch (\Exception $e) {
            throw new Engine_Exception(clearos_exception_message($e), CLEAROS_ERROR);
        }
    }

    /**
     * Acknowledge an SDN notification.
     *
     * @param int $id ID
     *
     * @return Object JSON-encoded response
     * @throws Webservice_Exception
     */

    public function acknowledge_sdn_notice($id)
    {
        clearos_profile(__METHOD__, __LINE__);

        try {

            $extras = array('id' => $id);

            $result = $this->request('registration', 'acknowledge_sdn_notice', $extras);

            return $result;
        } catch (Exception $e) {
            throw new Webservice_Exception(clearos_exception_message($e), CLEAROS_ERROR);
        }
    }

    /**
     * Get local registration flag.
     *
     * @return boolean true if registered
     *
     */

    public function get_local_registration_status()
    {
        clearos_profile(__METHOD__, __LINE__);

        try {

            $file = new File(self::FILE_REGISTERED_FLAG);
            if ($file->exists())
                return TRUE;
            return FALSE;
        } catch (Exception $e) {
            return FALSE;
        }
    }

    ///////////////////////////////////////////////////////////////////////////////
    // P R I V A T E   M E T H O D S
    ///////////////////////////////////////////////////////////////////////////////

    /**
     * Loads configuration files.
     *
     * @return void
     * @throws Engine_Exception
     */

    protected function _load_config()
    {
        clearos_profile(__METHOD__, __LINE__);

        $configfile = new Configuration_File(self::FILE_CONFIG);

        try {
            $this->config = $configfile->load();
        } catch (Exception $e) {
            throw new Engine_Exception(clearos_exception_message($e), CLEAROS_ERROR);
        }

        $this->is_loaded = TRUE;
    }

    /**
     * Generic set routine.
     *
     * @param string $key   key name
     * @param string $value value for the key
     *
     * @return  void
     * @throws Engine_Exception
     */

    protected function _set_parameter($key, $value)
    {
        clearos_profile(__METHOD__, __LINE__);

        try {
            $file = new File(self::FILE_CONFIG, TRUE);

            if (!$file->exists())
                $file->create('webconfig', 'webconfig', '0644');

            $match = $file->replace_lines("/^$key\s*=\s*/", "$key=$value\n");

            if (!$match)
                $file->add_lines("$key=$value\n");
        } catch (Exception $e) {
            throw new Engine_Exception(clearos_exception_message($e), CLEAROS_ERROR);
        }

        $this->is_loaded = FALSE;
    }

    /**
     * Generate a hardware specific ID.
     *
     *
     * @return String
     * @throws Engine_Exception
     */

    private function _get_hardware_id()
    {
        clearos_profile(__METHOD__, __LINE__);
        $hardware_id = "NA";

        $shell = new Shell();
        try {
            $shell->execute(self::COMMAND_DMIDECODE, "| grep 'ID:' | uniq | sed -e 's/.*ID://;s/ //g;s/-//g' | xargs echo -n | md5sum | awk '{ print $1 }'", TRUE);
            $hardware_id = $shell->get_last_output_line();
        } catch (\Exception $e) {
            // Do nothing
        }
        return $hardware_id;
    }

    /**
     * Fetch a build ID.
     *
     * On many systems, this may not exist.  The build ID is for those hardware vendors (system builders) that have deployed with
     * ClearOS pre-installed.
     *
     * @return String
     * @throws Engine_Exception
     */

    private function _get_build_id()
    {
        clearos_profile(__METHOD__, __LINE__);
        $build_id = "-";

        $file = new File(self::FILE_BUILD_ID, TRUE);
        try {
            if ($file->exists())
                $build_id = $file->get_contents();
        } catch (\Exception $e) {
            // Do nothing
        }
        return $build_id;
    }

    /**
     * Fetch a host ID.
     *
     * On many systems, this may not exist.  The host ID is for those systems deloyed in supported virtually hosted environments, like ClearVM
     *
     * @return String
     * @throws Engine_Exception
     */

    private function _get_host_id()
    {
        clearos_profile(__METHOD__, __LINE__);
        $host_id = "-";

        $file = new File(self::FILE_HOST_ID, TRUE);
        try {
            if ($file->exists())
                $host_id = $file->get_contents();
        } catch (\Exception $e) {
            // Do nothing
        }
        return $host_id;
    }

    /**
     * Fetch DMI system information.
     *
     *
     * @return String
     * @throws Engine_Exception
     */

    private function _get_dmi_system_info()
    {
        clearos_profile(__METHOD__, __LINE__);

        $shell = new Shell();
        $options['env'] = 'LANG=en_US';
        try {
            $shell->execute(self::COMMAND_DMIDECODE, NULL, TRUE, $options);
            $lines = $shell->get_output();
            $found_start = FALSE;
            $system_info = array();
            foreach ($lines as $line) {
                if (preg_match("/^System Information$/", $line)) {
                    $found_start = TRUE;
                    continue;
                }
                if (preg_match("/^[^\s].*/", $line) && $found_start)
                    break;
                if (preg_match("/^\s*(.*):\s(.*)/", $line, $match) && $found_start)
                    $system_info[$match[1]] = $match[2];
            }
        } catch (\Exception $e) {
            // Do nothing
        }
        return json_encode($system_info);
    }

    ///////////////////////////////////////////////////////////////////////////////
    // V A L I D A T I O N   R O U T I N E S
    ///////////////////////////////////////////////////////////////////////////////

    /**
     * Validation routine for sdn_username
     *
     * @param string $sdn_username SDN Username
     *
     * @return boolean TRUE if sdn_username is valid
     */

    public function validate_sdn_username($sdn_username)
    {
        clearos_profile(__METHOD__, __LINE__);

        if (! preg_match("/^[A-Za-z0-9]+$/", $sdn_username))
            return lang('base_username_invalid');

        if (strlen($sdn_username) < 4)
            return lang('base_username_too_short') . ' (4).';
    }

    /**
     * Validation routine for sdn_password
     *
     * @param string $sdn_password max instances
     *
     * @return boolean TRUE if sdn_password is valid
     */

    public function validate_sdn_password($sdn_password)
    {
        clearos_profile(__METHOD__, __LINE__);

        if (! preg_match("/^[A-Za-z0-9\\!\\@\\#\\$\\%\\^\\*\\(\\)\\-\\_\\&]+$/", $sdn_password))
            return lang('base_password_is_invalid');

        if (strlen($sdn_password) < 4)
            return lang('base_password_too_short') . ' (4).';
    }

    /**
     * Validation routine for system_name
     *
     * @param string $system_name system_name
     *
     * @return boolean TRUE if system_name is valid
     */

    public function validate_system_name($system_name)
    {
        clearos_profile(__METHOD__, __LINE__);

        if (! preg_match("/^[A-Za-z0-9\\ \\-\\_\\(\\)\\#\\.\\@]+$/", $system_name))
            return lang('registration_system_name_is_invalid');
    }

    /**
     * Validation registration type.
     *
     * @param int $type registration type
     *
     * @return mixed void if type is valid, errmsg otherwise
     */

    public function validate_registration_type($type)
    {
        clearos_profile(__METHOD__, __LINE__);

        if (FALSE)
            return lang('registration_type_is_invalid');
    }

    /**
     * Validation system for registration.
     *
     * @param int $system system
     *
     * @return mixed void if system is valid, errmsg otherwise
     */

    public function validate_system($system)
    {
        clearos_profile(__METHOD__, __LINE__);

        if ($system == 0)
            return lang('registration_select_system_required');
    }

    /**
     * Validation subscription for registration.
     *
     * @param int $subscription subscription
     *
     * @return mixed void if subscription is valid, errmsg otherwise
     */

    public function validate_subscription($subscription)
    {
        clearos_profile(__METHOD__, __LINE__);

        if ($subscription == 0)
            return lang('registration_select_subscription_required');
    }

    /**
     * Validation routine for email.
     *
     * @param string $email email
     *
     * @return mixed void if email is valid, errmsg otherwise
     */

    public function validate_email($email)
    {
        clearos_profile(__METHOD__, __LINE__);

        if (!preg_match("/^([a-zA-Z0-9])+([a-zA-Z0-9\._-])*@([a-zA-Z0-9_-])+([a-zA-Z0-9\._-]+)+$/", $email))
            return lang('base_email_address_invalid');
    }

    /**
     * Validation routine for country.
     *
     * @param string $country country
     *
     * @return string error message if country is invalid
     */

    public function validate_country($country)
    {
        clearos_profile(__METHOD__, __LINE__);

        $country_object = new Country();
        $country_list = $country_object->get_list();

        if (($country != '__') && (! array_key_exists($country, $country_list)))
            return lang('base_country_invalid');
    }

    /**
     * Validation routine for environment.
     *
     * @param string $environment environment
     *
     * @return string error message if environment is invalid
     */

    public function validate_environment($environment)
    {
        clearos_profile(__METHOD__, __LINE__);

        if ($environment == '0' || !preg_match('/\w+/', $environment))
            return lang('registration_environment_invalid');
    }

    /**
     * Validation routine for mailer.
     *
     * @param string $mailer mailer
     *
     * @return string error message if mailer is invalid
     */

    public function validate_mailer($mailer)
    {
        clearos_profile(__METHOD__, __LINE__);
    }

    /**
     * Validation routine for interest group.
     *
     * @param string $interest interest
     *
     * @return string error message if interest group is invalid
     */

    public function validate_interest_new_release($interest)
    {
        clearos_profile(__METHOD__, __LINE__);
    }

    /**
     * Validation routine for interest group.
     *
     * @param string $interest interest
     *
     * @return string error message if interest group is invalid
     */

    public function validate_interest_new_apps($interest)
    {
        clearos_profile(__METHOD__, __LINE__);
    }

    /**
     * Validation routine for interest group.
     *
     * @param string $interest interest
     *
     * @return string error message if interest group is invalid
     */

    public function validate_interest_betas($interest)
    {
        clearos_profile(__METHOD__, __LINE__);
    }

    /**
     * Validation routine for interest group.
     *
     * @param string $interest interest
     *
     * @return string error message if interest group is invalid
     */

    public function validate_interest_promotions($interest)
    {
        clearos_profile(__METHOD__, __LINE__);
    }
}
